﻿--[[
HideParty.lua

This file is part of HideParty. <http://www.curse.com/addons/wow/hideparty>

Copyright © 2010-2013 Matthew Casey <mdcasey@chabloom.com>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
]]
HideParty = LibStub("AceAddon-3.0"):NewAddon("HideParty", "AceConsole-3.0", "AceEvent-3.0")
local HIDEPARTY_VERSION = GetAddOnMetadata("HideParty", "Version")
local L = LibStub("AceLocale-3.0"):GetLocale("HideParty")

local options = {
	name = "HideParty",
	handler = HideParty,
	type = 'group',
	args = {
		hide = {
			name = L["GENERAL"],
			handler = HideParty,
			type = 'group',
			args = {
				buff = {
					type = 'toggle',
					name = L["BUFF"],
					desc = L["BUFF_DESC"],
					get = 'GetFrame',
					set = 'SetFrame'
				},
				cast = {
					type = 'toggle',
					name = L["CAST"],
					desc = L["CAST_DESC"],
					get = 'GetFrame',
					set = 'SetFrame'
				},
				group = {
					type = 'toggle',
					name = L["GROUP"],
					desc = L["GROUP_DESC"],
					get = 'GetFrame',
					set = 'SetFrame'
				},
				party = {
					type = 'toggle',
					name = L["PARTY"],
					desc = L["PARTY_DESC"],
					get = 'GetFrame',
					set = 'SetFrame'
				},
				player = {
					type = 'toggle',
					name = L["PLAYER"],
					desc = L["PLAYER_DESC"],
					get = 'GetFrame',
					set = 'SetFrame'
				},
				target = {
					type = 'toggle',
					name = L["TARGET"],
					desc = L["TARGET_DESC"],
					get = 'GetFrame',
					set = 'SetFrame'
				},
			},
		},
	},
}

local show = {
	buff,
	cast,
	group,
	party,
	player,
	target
}
local hide = {
	buff,
	cast,
	group,
	party,
	player,
	target
}
local hidden = {
	buff,
	cast,
	group,
	party,
	player,
	target
}

local defaults = {
	profile = {
		buff = false,
		cast = false,
		group = true,
		party = true,
		player = true,
		target = true
	},
}

local isPrinted = false

function HideParty:GetFrame(info, value)
	return self.db.profile[info[#info]]
end

function HideParty:SetFrame(info, value)
	self.db.profile[info[#info]] = value
	self:UpdateFrames()
end

local function hook_compactparty()
	hooksecurefunc("CompactPartyFrame_UpdateShown",function()
		if hidden["party"] then
			hide["party"]()
		end
	end)
	hook_compactparty = nil
end

local function hook_playerframe()
	hooksecurefunc("PlayerFrame_HideVehicleTexture",function()
		if hidden["runebar"] then
			hide["runebar"]()
		end
	end)
	hook_playerframe = nil
end

local function hook_raidmanager()
	hooksecurefunc("CompactRaidFrameManager_UpdateShown",function()
		if hidden["raid"] then
			hide["raid"]()
		end
	end)
	hook_raidmanager = nil
end

function hide:runebar()
	if hook_playerframe then
		hook_playerframe()
	end
	RuneFrame:UnregisterAllEvents()
	RuneFrame:Hide()
end

function show:runebar()
	local _,class = UnitClass("player")
	if class == "DEATHKNIGHT" then
		RuneFrame:Show()
	end
	RuneFrame:GetScript("OnLoad")(RuneFrame)
	RuneFrame:GetScript("OnEvent")(RuneFrame, "PLAYER_ENTERING_WORLD")
end

function hide:buff()
	BuffFrame:Hide()
	TemporaryEnchantFrame:Hide()
	ConsolidatedBuffs:Hide()
	BuffFrame:UnregisterAllEvents()
end

function show:buff()
	BuffFrame:Show()
	if GetCVarBool("consolidateBuffs") then
		ConsolidatedBuffs:Show()
	end
	TemporaryEnchantFrame:Show()
	BuffFrame:RegisterEvent("UNIT_AURA")
end

function hide:cast()
	CastingBarFrame:UnregisterAllEvents()
	PetCastingBarFrame:UnregisterAllEvents()
end

function show:cast()
	CastingBarFrame:GetScript("OnLoad")(CastingBarFrame)
	PetCastingBarFrame:GetScript("OnLoad")(PetCastingBarFrame)
end

function hide:player()
	PlayerFrame:Hide()
	hide:runebar()
end

function show:player()
	PlayerFrame:Show()
	show:runebar()
end

function hide:party()
	for i = 1, 4 do
		local frame = _G["PartyMemberFrame"..i]
		frame:UnregisterAllEvents()
		frame:Hide()
		frame.Show = function() end
	end
	
	UIParent:UnregisterEvent("GROUP_ROSTER_UPDATE")
end

function show:party()
	for i = 1, 4 do
		local frame = _G["PartyMemberFrame"..i]
		frame.Show = nil
		frame:GetScript("OnLoad")(frame)
		frame:GetScript("OnEvent")(frame, "GROUP_ROSTER_UPDATE")
		
		PartyMemberFrame_UpdateMember(frame)
	end

	UIParent:RegisterEvent("GROUP_ROSTER_UPDATE")
end

local raidManager
function hide:raid()
	CompactRaidFrameManager:UnregisterEvent("GROUP_ROSTER_UPDATE")
	CompactRaidFrameManager:UnregisterEvent("PLAYER_ENTERING_WORLD")
	CompactRaidFrameManager:Hide()
	raidManager = CompactRaidFrameManager_GetSetting("IsShown")
	if raidManager and raidManager ~= "0" then 
		CompactRaidFrameManager_SetSetting("IsShown", "0")
	end
	if hook_raidmanager then
		hook_raidmanager()
	end
end

function show:raid()
	CompactRaidFrameManager:RegisterEvent("GROUP_ROSTER_UPDATE")	
	CompactRaidFrameManager:RegisterEvent("PLAYER_ENTERING_WORLD")
	if GetDisplayedAllyFrames then
		if GetDisplayedAllyFrames() == "raid" then
			CompactRaidFrameManager:Show()
		end
	elseif GetNumRaidMembers() > 0 then
		CompactRaidFrameManager:Show()
	end
	if raidManager and raidManager ~= "0" then
		CompactRaidFrameManager_SetSetting("IsShown", "1")
	end
end

function hide:target()
	TargetFrame:UnregisterAllEvents()
	TargetFrame:Hide()

	ComboFrame:UnregisterAllEvents()
end

function show:target()
	TargetFrame:GetScript("OnLoad")(TargetFrame)

	ComboFrame:GetScript("OnLoad")(ComboFrame)
end

function HideParty:UpdateFrames()
	if InCombatLockdown() then
		if not isPrinted then
			self:Print("Disabled during combat")
			isPrinted = true;
		end
	else
		for frame in pairs(show) do
			if self:IsEnabled() and self.db.profile[frame] then
				if not hidden[frame] then
					hidden[frame] = true
					hide[frame](self)
				end
			else
				if hidden[frame] then
					hidden[frame] = nil
					show[frame](self)
				end
			end
		end
	end
end

function HideParty:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("HideParty50DB", defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateFrames")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateFrames")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateFrames")
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("HideParty", options)
	self.Options = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("HideParty")
end

function HideParty:OnEnable()
	self:RegisterChatCommand("HP", "SlashCommand")
	self:RegisterChatCommand("HIDEPARTY", "SlashCommand")
	self:ShowVersion()
	self:UpdateFrames()
end

function HideParty:ShowVersion()
	self:Print("HideParty version", HIDEPARTY_VERSION)
end

function HideParty:SlashCommand(input)
	if input == "version" then
		self:ShowVersion()
	elseif input == "options" then
		InterfaceOptionsFrame_OpenToCategory(self.Options)
	else
		self:Print("/hp options")
		self:Print("/hp version")
	end
end